package xemail

import (
	"crypto/rand"
	"crypto/tls"
	"fmt"
	"gitlab.wanzhuangkj.com/tush/xpkg/xerrors/xerror"
	"math/big"

	"gopkg.in/gomail.v2"
)

// 配置结构体 (建议从环境变量或配置文件中读取)
type Config struct {
	SMTPServer   string
	SMTPPort     int
	SMTPUsername string
	SMTPPassword string
	FromEmail    string
}

func GenerateVerificationCode(length int) (string, error) {
	code := ""
	for i := 0; i < length; i++ {
		num, err := rand.Int(rand.Reader, big.NewInt(10))
		if err != nil {
			return "", err
		}
		code += num.String()
	}
	return code, nil
}

func SendVerificationCode(cfg Config, toEmail string, code string) error {
	m := gomail.NewMessage()
	m.SetHeader("From", cfg.FromEmail)
	m.SetHeader("To", toEmail)
	m.SetHeader("Subject", "[万桩严选商城]验证码")
	m.SetBody("text/html", fmt.Sprintf(`
		<html>
		<body>
			<h3>验证码通知</h3>
			<p>您的验证码是：<strong>%s</strong></p>
			<p>有效期15分钟，请勿泄露给他人</p>
		</body>
		</html>
	`, code))
	d := gomail.NewDialer(
		cfg.SMTPServer,
		cfg.SMTPPort,
		cfg.SMTPUsername,
		cfg.SMTPPassword,
	)
	d.TLSConfig = &tls.Config{ServerName: cfg.SMTPServer}
	d.SSL = false
	if err := d.DialAndSend(m); err != nil {
		return xerror.New(err.Error())
	}
	return nil
}
